from .consts import *
        
class SleepIQSleeper:
    def __init__(self, api, bed_id, sleeper_id, side):
        self.api = api
        self.bed_id = bed_id
        self.sleeper_id = sleeper_id
        self.side = SIDES[side]
        self.side_full = SIDES_FULL[side]
        self.active = False
        self.name = ""
        
        self.in_bed = False
        self.pressure = 0
        self.sleep_number = 0
        self.fav_sleep_number = 0
        
        
    def __str__(self):
        return f"SleepIQSleeper[{self.side}]({self.name}, in_bed={self.in_bed}, sn={self.sleep_number})" 
    def __repr__(self):
        return f"SleepIQSleeper[{self.side}]({self.name}, in_bed={self.in_bed}, sn={self.sleep_number})" 

    # Calibrate or "baseline" bed
    async def calibrate(self):
        await self.api.put('sleeper/'+self.sleeper_id+'/calibrate')

    # Set sleep number 5-100 (multiple of 5)
    #  Note: This function does not work on all models, even if sleep number can be fetched
    async def set_sleepnumber(self, setting):
        if 0 > setting or setting > 100 :
            raise ValueError("Invalid SleepNumber, must be between 0 and 100")
        setting = int(round(setting/5))*5
        data = {"sleepNumber": setting, 'side': self.side}
        await self.api.put('bed/'+self.bed_id+'/sleepNumber', data)


    # Set favorite sleep number 5-100 (multiple of 5)
    async def set_favsleepnumber(self, setting):
        if 0 > setting or setting > 100:
            raise ValueError("Invalid SleepNumber, must be between 0 and 100")
        setting = int(round(setting/5))*5
        data = {'side': self.side, "sleepNumberFavorite": setting}
        await self.api.put('bed/'+self.bed_id+'/sleepNumberFavorite', data)
        await self.update_favsleepnumber()


    async def fetch_favsleepnumber(self):
        json = await self.api.get('bed/'+self.bed_id+'/sleepNumberFavorite')
        self.fav_sleep_number = json['sleepNumberFavorite'+ self.side_full]





from .consts import ACTUATORS, ACTUATORS_FULL, SIDES, SIDES_FULL

class SleepIQActuator:
    def __init__(self, api, bed_id, side, actuator):
        self._api = api
        self.bed_id = bed_id
        self.side = SIDES[side] if side else None
        self.side_full = SIDES_FULL[side] if side else None
        self.actuator = ACTUATORS[actuator]
        self.actuator_full = ACTUATORS_FULL[actuator]
        self.position = 0

    def __str__(self):
        return f"SleepIQActuator[{self.actuator_full} {self.side_full}], position={self.position}"
    def __repr__(self):
        return f"SleepIQActuator[{self.actuator_full} {self.side_full}], position={self.position}"

    async def set_position(self, position, slowSpeed=False):
        if position < 0 or position > 100:
            raise ValueError("Invalid position, must be between 0 and 100")
        if position == self.position:
            return
        data = {"position": position, "side": self.side if self.side else "R", "actuator": self.actuator, "speed": 1 if slowSpeed else 0}
        await self._api.put(f"bed/{self.bed_id}/foundation/adjustment/micro", data)

    async def update(self, data):
        # For non-split actuators, it doesn't matter which side we get the
        # value from, it'll always be the same for either
        side_full = self.side_full if self.side_full else "Right"

        # The API reports position in hex, but is set with an integer.
        # We'll always show position with an integer value.
        self.position = int(data[f"fs{side_full}{self.actuator_full}Position"], 16)


from .sleeper import *
from .foundation import *
from .consts import *

class SleepIQBed:
    def __init__(self, api, data):
        self._api = api
        
        self.name = data['name']
        self.id = data['bedId']
        self.mac_addr = data['macAddress']
        self.paused = False
        self.sleepers = [
            SleepIQSleeper(api, self.id, data['sleeperLeftId'], 0),
            SleepIQSleeper(api, self.id, data['sleeperRightId'], 1),
        ]
        self.foundation = SleepIQFoundation(api, self.id)
        
        self.model = "Unknown"
        if 'model' in data:
            self.model = data['model']
        elif 'components' in data:
            for comp in data['components']:
                if comp['base'] == "BASE":
                    self.model = comp['model']
                    break
       
    def __str__(self):
        return f"SleepIQBed({self.name}, model={self.model}, id={self.id}) "+str(self.sleepers)+" "+str(self.foundation)
    def __repr__(self):
        return f"SleepIQBed({self.name}, model={self.model}, id={self.id}) "+str(self.sleepers)+" "+str(self.foundation)
        
    # Calibrate or "baseline" bed
    async def calibrate(self):
        for sleeper in self.sleepers:
            if sleeper.sleeper_id:
                await sleeper.calibrate()
                break
        
    async def stop_pump(self):
        await self._api.put('bed/'+self.id+'/pump/forceIdle')
        
    async def fetch_pause_mode(self):
        json = await self._api.get('bed/'+self.id+'/pauseMode')
        self.paused = json.get('pauseMode',"") == "on"
        
    async def set_pause_mode(self, mode):
        params = {'mode': "on" if mode else "off"}
        await self._api.put('bed/'+self.id+'/pauseMode', params=params)
        self.paused = mode
        


from .consts import BED_LIGHTS, BED_PRESETS, FOUNDATION_TYPES, MASSAGE_MODE, MASSAGE_SPEED
from .actuator import SleepIQActuator
from .light import SleepIQLight

class SleepIQFoundation:
    def __init__(self, api, bed_id):
        self._api = api
        self.bed_id = bed_id
        self.lights = []
        self.features = {}
        self.type = None
        self.actuators = []
        self.isMoving = False
        self.preset = ""
        
    def __str__(self):
        return f"SleepIQFoundation[{self.type}](lights: {len(self.lights)}, features: {len(self.features)}, actuators: {len(self.actuators)})" 
    def __repr__(self):
        return f"SleepIQFoundation[{self.type}](lights: {len(self.lights)}, features: {len(self.features)}, actuators: {len(self.actuators)})" 
        
    async def init_lights(self):
        for light in BED_LIGHTS:
            exists = await self._api.check(f"bed/{self.bed_id}/foundation/outlet", params={"outletId": light})
            if exists:
                self.lights.append(SleepIQLight(self._api, self.bed_id, light))
        await self.update_lights()

    async def update_lights(self):
        for light in self.lights:
            await light.update()

    async def init_actuators(self):
        if not self.type:
            return
        data = await self._api.get(f"bed/{self.bed_id}/foundation/status")
        self.isMoving = data["fsIsMoving"]

        if self.type in ["single", "easternKing"]:
            self.actuators = [
                SleepIQActuator(self._api, self.bed_id, None, 0),
                SleepIQActuator(self._api, self.bed_id, None, 1)
            ]
        elif self.type == "splitHead":
            self.actuators = [
                SleepIQActuator(self._api, self.bed_id, 0, 0),
                SleepIQActuator(self._api, self.bed_id, 1, 0),
                SleepIQActuator(self._api, self.bed_id, None, 1)
            ]
        else:
            self.actuators = [
                SleepIQActuator(self._api, self.bed_id, 0, 0),
                SleepIQActuator(self._api, self.bed_id, 1, 0),
                SleepIQActuator(self._api, self.bed_id, 0, 1),
                SleepIQActuator(self._api, self.bed_id, 1, 1)
            ]

        for actuator in self.actuators:
            await actuator.update(data)

    async def update_actuators(self):
        if not self.type:
            return
        data = await self._api.get(f"bed/{self.bed_id}/foundation/status")
        self.isMoving = data["fsIsMoving"]
        for actuator in self.actuators:
            await actuator.update(data)

    async def fetch_features(self):
        have_foundation = await self._api.check('bed/'+self.bed_id+'/foundation/system')
        if not have_foundation:
           self.type = None
           return           
                
        fs = await self._api.get('bed/'+self.bed_id+'/foundation/system')
        features_flags = fs.get('fsBoardFeatures')
        self.features['boardIsASingle'] = features_flags & (1 < 0)
        self.features['hasMassageAndLight'] = features_flags & (1 < 1)
        self.features['hasFootControl'] = features_flags & (1 < 2)
        self.features['hasFootWarming'] = features_flags & (1 < 3)
        self.features['hasUnderbedLight'] = features_flags & (1 < 4)
        self.type = FOUNDATION_TYPES[fs.get('fsBedType')]
        
        self.features['leftUnderbedLightPMW'] = fs.get('fsLeftUnderbedLightPWM')
        self.features['rightUnderbedLightPMW'] = fs.get('fsRightUnderbedLightPWM')

        if 'hasMassageAndLight' in self.features:
            self.features['hasUnderbedLight'] = True
        if 'split' in self.type:
            self.features['boardIsASingle'] = False
            
        
    async def stop_motion(self, side):
        data = {"footMotion":1, "headMotion":1, "massageMotion":1, "side":side}
        await self._api.put('bed/'+self.bed_id+'/foundation/motion', data)

    async def set_preset(self, side, preset, slowSpeed=False):
        #
        # preset 1-6
        # slowSpeed False=fast, True=slow
        #
        if preset not in BED_PRESETS:
            raise ValueError("Invalid preset")
            
        data = {'preset':preset,'side':side,'speed':1 if slowSpeed else 0}
        await self._api.put('bed/'+self.bed_id+'/foundation/preset', data)

    async def set_foundation_massage(self, side, footSpeed, headSpeed, timer=0, mode=0):
        #
        # footSpeed 0-3
        # headSpeed 0-3
        # mode 0-3
        #
        if mode not in MASSAGE_MODE:
            raise ValueError("Invalid mode")
        if mode != 0:
            footSpeed = 0
            headSpeed = 0
        if not all(speed in MASSAGE_SPEED for speed in [footSpeed, headSpeed]):
            raise ValueError("Invalid head or foot speed")
        
        data = {'footMassageMotor':footSpeed,
                'headMassageMotor':headSpeed,
                'massageTimer':timer,
                'massageWaveMode':mode,
                'side':side}
        await self._api.put('bed/'+self.bed_id+'/foundation/adjustment', data)



class SleepIQLight:
    def __init__(self, api, bed_id, outlet_id):
        self._api = api
        self.bed_id = bed_id
        self.is_on = False
        self.outlet_id = outlet_id

    def __str__(self):
        return f"SleepIQLight[{self.outlet_id}]: {'On' if self.is_on else 'Off'}"
    def __repr__(self):
        return f"SleepIQLight[{self.outlet_id}]: {'On' if self.is_on else 'Off'}"

    async def turn_on(self):
        await self.set_light(True)
        self.is_on = True

    async def turn_off(self):
        await self.set_light(False)
        self.is_on = False

    async def set_light(self, on):
        data = {"outletId": self.outlet_id, "setting": 1 if on else 0}
        await self._api.put(f"bed/{self.bed_id}/foundation/outlet", data)

    async def update(self):
        params = {"outletId": self.outlet_id}
        status = await self._api.get(f"bed/{self.bed_id}/foundation/outlet", params=params)
        self.is_on = status["setting"] == 1


import aiohttp
import asyncio
import random
from .consts import *
from .sleeper import *
from .bed import *

class SleepIQLoginException(Exception):
    pass
class SleepIQTimeoutException(Exception):
    pass
class SleepIQAPIException(Exception):
    pass

# Return a randomly generated sorta valid User Agent string
def random_user_agent():
    uas = {
        "Edge": ("AppleWebKit/537.36 (KHTML, like Gecko) "
                 "Chrome/98.0.4758.80 Safari/537.36 Edg/98.0.1108.43"),
        "Chrome": ("AppleWebKit/537.36 (KHTML, like Gecko) "
                   "Chrome/97.0.4692.99 Safari/537.36"),
        "Firefox": "Gecko/20100101 Firefox/96.0",
        "iphone": ("AppleWebKit/605.1.15 (KHTML, like Gecko) "
                   "Version/15.2 Mobile/15E148 Safari/604.1"),
        "Safari": ("AppleWebKit/605.1.15 (KHTML, like Gecko) "
                   "Version/11.1.2 Safari/605.1.15"),
    }
    os = {
        "windows": "Windows NT 10.0; Win64; x64",
        "iphone": "iPhone; CPU iPhone OS 15_2_1 like Mac OS X",
        "mac": "Macintosh; Intel Mac OS X 10_11_6",
    }
    template = "Mozilla/5.0 ({os}) {ua}"

    return template.format(
        os=random.choice(list(os.values())),
        ua=random.choice(list(uas.values()))
    )

class AsyncSleepIQ:
    def __init__(self, email=None, password=None, login_method=LOGIN_KEY, client_session=None):
        self._email = email
        self._password = password
        self._session = client_session or aiohttp.ClientSession()
        self._headers = {'User-Agent': random_user_agent()}
        self._key = ""
        self._login_method = login_method
        
        self.beds = []

    async def login(self, email="", password=""):            
        if not email: email = self._email
        if not password: password = self._password
        if not email or not password:
            raise SleepIQLoginException("username/password not set")
            
        try:
            if self._login_method == LOGIN_KEY:
                await self.login_key(email, password)
            else:
                await self.login_cookie(email, password)

        except asyncio.TimeoutError as ex:
            # timed out
            raise SleepIQTimeoutException("API call timed out") from ex
        except SleepIQTimeoutException as ex:
            raise ex
        except Exception as ex:
            raise SleepIQLoginException(f"Connection failure: {ex}") from ex
            
        # store in case we need to login again  
        self._email = email
        self._password = password    

    async def login_key(self, email="", password=""):
        self._key = ""
        auth_data = {'login': email, 'password': password}
            
            
        async with self._session.put(
            API_URL+'/login', headers=self._headers, timeout=TIMEOUT, json=auth_data
        ) as resp:

            if resp.status == 401:
                raise SleepIQLoginException("Incorrect username or password")
            if resp.status == 403:
                raise SleepIQLoginException("User Agent is blocked. May need to update GenUserAgent data?")
            if resp.status not in (200, 201):
                raise SleepIQLoginException("Unexpected response code: {code}\n{body}".format(
                    code=resp.status,
                    body=resp.text,
                ))
                
            json = await resp.json()
            self._key = json['key']          

    async def login_cookie(self, email="", password=""):
        auth_data = {'Email': email, 'Password': password, 'ClientID': "2oa5825venq9kek1dnrhfp7rdh"}
        async with self._session.post(
            "https://l06it26kuh.execute-api.us-east-1.amazonaws.com/Prod/v1/token", 
            headers=self._headers, timeout=TIMEOUT, json=auth_data
        ) as resp:

            if resp.status == 401:
                raise SleepIQLoginException("Incorect username or password")
            if resp.status == 403:
                raise SleepIQLoginException("User Agent is blocked. May need to update GenUserAgent data?")
            if resp.status not in (200, 201):
                raise SleepIQLoginException("Unexpected response code: {code}\n{body}".format(
                    code=resp.status,
                    body=resp.text,
                ))
            json = await resp.json()
            token = json['data']['AccessToken']
            self._headers["Authorization"] = token
            
        async with self._session.get(
            API_URL+'/user/jwt', headers=self._headers, timeout=TIMEOUT
        ) as resp:
            if resp.status not in (200, 201):
                raise SleepIQLoginException("Unexpected response code: {code}\n{body}".format(
                    code=resp.status,
                    body=resp.text,
                ))           

    async def close_session(self):
        if self._session:
            await self._session.close()

    async def put(self, url, data="", params={}):
        await self.__make_request(self._session.put, url, data, params)
        
    async def get(self, url, data="", params={}):
        return await self.__make_request(self._session.get, url, data, params)
        
    async def check(self, url, data="", params={}):
        return await self.__make_request(self._session.get, url, data, params, check=True)
    
    async def __make_request(self, make_request, url, json={}, params={}, retry=True, check=False):
        timeout = aiohttp.ClientTimeout(total=TIMEOUT)
        params['_k'] = self._key
        try:
            async with make_request(
                API_URL + "/" + url, headers=self._headers, timeout=timeout, json=json, params=params
            ) as resp:
                if resp.status != 200:
                    if resp.status == 404 and check:
                        return False
                        
                    if retry and resp.status in (401, 404):
                        # login and try again
                        await self.login()
                        return await self.__make_request(make_request, url, json, params, False)
                    raise SleepIQAPIException(f"API call error response {resp.status}\n{resp.text}")
                if check:
                    return True
                return await resp.json()
        except asyncio.TimeoutError as ex:
            # timed out
            raise SleepIQTimeoutException("API call timed out") from ex
        
    # initialize beds and sleepers from API
    async def init_beds(self):             
        data = await self.get('bed')
        
        # get beds
        self.beds = {bed['bedId']: SleepIQBed(self, bed) for bed in data['beds']}
        
        # get sleepers and assign to beds
        data = await self.get('sleeper')
        for sleeper_data in data['sleepers']:
            if sleeper_data['bedId'] not in self.beds: continue
            sleeper = self.beds[sleeper_data['bedId']].sleepers[sleeper_data['side']]
            sleeper.name = sleeper_data['firstName']
            sleeper.active = sleeper_data['active']
            
        # init foundations
        for bed in self.beds.values():
            await bed.foundation.fetch_features()
            await bed.foundation.init_actuators()
            await bed.foundation.init_lights()
        
    # update statuses of sleepers/beds
    async def fetch_bed_statuses(self):
        data = await self.get('bed/familyStatus')
        for bed_status in data['beds']:
            if bed_status['bedId'] not in self.beds: continue
            for i, side in enumerate(["left", "right"]):
                self.beds[bed_status['bedId']].sleepers[i].in_bed = bed_status[side+'Side']['isInBed']
                self.beds[bed_status['bedId']].sleepers[i].pressure = bed_status[side+'Side']['pressure']
                self.beds[bed_status['bedId']].sleepers[i].sleep_number = bed_status[side+'Side']['sleepNumber']




API_URL = "https://prod-api.sleepiq.sleepnumber.com/rest"
TIMEOUT = 10

LOGIN_KEY=1
LOGIN_COOKIE=2

RIGHT_NIGHT_STAND = 1
LEFT_NIGHT_STAND = 2
RIGHT_NIGHT_LIGHT = 3
LEFT_NIGHT_LIGHT = 4

BED_LIGHTS = [
        RIGHT_NIGHT_STAND,
        LEFT_NIGHT_STAND,
        RIGHT_NIGHT_LIGHT,
        LEFT_NIGHT_LIGHT
    ]

FAVORITE = 1
READ = 2
WATCH_TV = 3
FLAT = 4
ZERO_G = 5
SNORE = 6

BED_PRESETS = [
        FAVORITE,
        READ,
        WATCH_TV,
        FLAT,
        ZERO_G,
        SNORE
    ]

OFF = 0
LOW = 1
MEDIUM = 2
HIGH = 3

MASSAGE_SPEED = [
        OFF,
        LOW,
        MEDIUM,
        HIGH
    ]

SOOTHE = 1
REVITILIZE = 2
WAVE = 3

MASSAGE_MODE = [
        OFF,
        SOOTHE,
        REVITILIZE,
        WAVE
    ]
SIDES = ['L', 'R']
SIDES_FULL= ['Left', 'Right']

FOUNDATION_TYPES = ['single', 'splitHead', 'splitKing','easternKing']

ACTUATORS = ['H', 'F']
ACTUATORS_FULL = ['Head', 'Foot']



from setuptools import setup
import asyncsleepiq

def readme():
    with open('README.md') as f:
        return f.read()

setup(name='asyncsleepiq',
      packages=['asyncsleepiq'],
      version=asyncsleepiq.__version__,
      description='ASync SleepIQ API',
      long_description=readme(),
      long_description_content_type="text/markdown",
      url='http://github.com/kbickar/asyncsleepiq',
      author='Keilin Bickar',
      author_email='trumpetgod@gmail.com',
      license='MIT',
      install_requires=[
          'aiohttp;python_version>="3.5"',
      ],
      classifiers = [
          'Programming Language :: Python :: 3',
      ],
)
